@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,12 +13,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import proto  # type: ignore
 
-
 from google.cloud.firestore_admin_v1.types import index as gfa_index
-from google.protobuf import timestamp_pb2 as timestamp  # type: ignore
+from google.protobuf import timestamp_pb2  # type: ignore
 
 
 __protobuf__ = proto.module(
@@ -73,16 +70,11 @@ class IndexOperationMetadata(proto.Message):
             The progress, in bytes, of this operation.
     """
 
-    start_time = proto.Field(proto.MESSAGE, number=1, message=timestamp.Timestamp,)
-
-    end_time = proto.Field(proto.MESSAGE, number=2, message=timestamp.Timestamp,)
-
-    index = proto.Field(proto.STRING, number=3)
-
+    start_time = proto.Field(proto.MESSAGE, number=1, message=timestamp_pb2.Timestamp,)
+    end_time = proto.Field(proto.MESSAGE, number=2, message=timestamp_pb2.Timestamp,)
+    index = proto.Field(proto.STRING, number=3,)
     state = proto.Field(proto.ENUM, number=4, enum="OperationState",)
-
     progress_documents = proto.Field(proto.MESSAGE, number=5, message="Progress",)
-
     progress_bytes = proto.Field(proto.MESSAGE, number=6, message="Progress",)
 
 
@@ -117,7 +109,6 @@ class FieldOperationMetadata(proto.Message):
 
     class IndexConfigDelta(proto.Message):
         r"""Information about an index configuration change.
-
         Attributes:
             change_type (google.cloud.firestore_admin_v1.types.FieldOperationMetadata.IndexConfigDelta.ChangeType):
                 Specifies how the index is changing.
@@ -136,23 +127,16 @@ class FieldOperationMetadata(proto.Message):
             number=1,
             enum="FieldOperationMetadata.IndexConfigDelta.ChangeType",
         )
-
         index = proto.Field(proto.MESSAGE, number=2, message=gfa_index.Index,)
 
-    start_time = proto.Field(proto.MESSAGE, number=1, message=timestamp.Timestamp,)
-
-    end_time = proto.Field(proto.MESSAGE, number=2, message=timestamp.Timestamp,)
-
-    field = proto.Field(proto.STRING, number=3)
-
+    start_time = proto.Field(proto.MESSAGE, number=1, message=timestamp_pb2.Timestamp,)
+    end_time = proto.Field(proto.MESSAGE, number=2, message=timestamp_pb2.Timestamp,)
+    field = proto.Field(proto.STRING, number=3,)
     index_config_deltas = proto.RepeatedField(
         proto.MESSAGE, number=4, message=IndexConfigDelta,
     )
-
     state = proto.Field(proto.ENUM, number=5, enum="OperationState",)
-
     progress_documents = proto.Field(proto.MESSAGE, number=6, message="Progress",)
-
     progress_bytes = proto.Field(proto.MESSAGE, number=7, message="Progress",)
 
 
@@ -181,19 +165,13 @@ class ExportDocumentsMetadata(proto.Message):
             Where the entities are being exported to.
     """
 
-    start_time = proto.Field(proto.MESSAGE, number=1, message=timestamp.Timestamp,)
-
-    end_time = proto.Field(proto.MESSAGE, number=2, message=timestamp.Timestamp,)
-
+    start_time = proto.Field(proto.MESSAGE, number=1, message=timestamp_pb2.Timestamp,)
+    end_time = proto.Field(proto.MESSAGE, number=2, message=timestamp_pb2.Timestamp,)
     operation_state = proto.Field(proto.ENUM, number=3, enum="OperationState",)
-
     progress_documents = proto.Field(proto.MESSAGE, number=4, message="Progress",)
-
     progress_bytes = proto.Field(proto.MESSAGE, number=5, message="Progress",)
-
-    collection_ids = proto.RepeatedField(proto.STRING, number=6)
-
-    output_uri_prefix = proto.Field(proto.STRING, number=7)
+    collection_ids = proto.RepeatedField(proto.STRING, number=6,)
+    output_uri_prefix = proto.Field(proto.STRING, number=7,)
 
 
 class ImportDocumentsMetadata(proto.Message):
@@ -221,19 +199,13 @@ class ImportDocumentsMetadata(proto.Message):
             The location of the documents being imported.
     """
 
-    start_time = proto.Field(proto.MESSAGE, number=1, message=timestamp.Timestamp,)
-
-    end_time = proto.Field(proto.MESSAGE, number=2, message=timestamp.Timestamp,)
-
+    start_time = proto.Field(proto.MESSAGE, number=1, message=timestamp_pb2.Timestamp,)
+    end_time = proto.Field(proto.MESSAGE, number=2, message=timestamp_pb2.Timestamp,)
     operation_state = proto.Field(proto.ENUM, number=3, enum="OperationState",)
-
     progress_documents = proto.Field(proto.MESSAGE, number=4, message="Progress",)
-
     progress_bytes = proto.Field(proto.MESSAGE, number=5, message="Progress",)
-
-    collection_ids = proto.RepeatedField(proto.STRING, number=6)
-
-    input_uri_prefix = proto.Field(proto.STRING, number=7)
+    collection_ids = proto.RepeatedField(proto.STRING, number=6,)
+    input_uri_prefix = proto.Field(proto.STRING, number=7,)
 
 
 class ExportDocumentsResponse(proto.Message):
@@ -249,7 +221,7 @@ class ExportDocumentsResponse(proto.Message):
             operation completes successfully.
     """
 
-    output_uri_prefix = proto.Field(proto.STRING, number=1)
+    output_uri_prefix = proto.Field(proto.STRING, number=1,)
 
 
 class Progress(proto.Message):
@@ -264,9 +236,8 @@ class Progress(proto.Message):
             The amount of work completed.
     """
 
-    estimated_work = proto.Field(proto.INT64, number=1)
-
-    completed_work = proto.Field(proto.INT64, number=2)
+    estimated_work = proto.Field(proto.INT64, number=1,)
+    completed_work = proto.Field(proto.INT64, number=2,)
 
 
 __all__ = tuple(sorted(__protobuf__.manifest))
