@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,7 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 from collections import OrderedDict
 import functools
 import re
@@ -31,10 +29,10 @@ from typing import (
 import pkg_resources
 
 import google.api_core.client_options as ClientOptions  # type: ignore
-from google.api_core import exceptions  # type: ignore
+from google.api_core import exceptions as core_exceptions  # type: ignore
 from google.api_core import gapic_v1  # type: ignore
 from google.api_core import retry as retries  # type: ignore
-from google.auth import credentials  # type: ignore
+from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
 from google.cloud.firestore_v1.services.firestore import pagers
@@ -44,9 +42,8 @@ from google.cloud.firestore_v1.types import document as gf_document
 from google.cloud.firestore_v1.types import firestore
 from google.cloud.firestore_v1.types import query
 from google.cloud.firestore_v1.types import write as gf_write
-from google.protobuf import timestamp_pb2 as timestamp  # type: ignore
-from google.rpc import status_pb2 as status  # type: ignore
-
+from google.protobuf import timestamp_pb2  # type: ignore
+from google.rpc import status_pb2  # type: ignore
 from .transports.base import FirestoreTransport, DEFAULT_CLIENT_INFO
 from .transports.grpc_asyncio import FirestoreGrpcAsyncIOTransport
 from .client import FirestoreClient
@@ -74,18 +71,14 @@ class FirestoreAsyncClient:
     parse_common_billing_account_path = staticmethod(
         FirestoreClient.parse_common_billing_account_path
     )
-
     common_folder_path = staticmethod(FirestoreClient.common_folder_path)
     parse_common_folder_path = staticmethod(FirestoreClient.parse_common_folder_path)
-
     common_organization_path = staticmethod(FirestoreClient.common_organization_path)
     parse_common_organization_path = staticmethod(
         FirestoreClient.parse_common_organization_path
     )
-
     common_project_path = staticmethod(FirestoreClient.common_project_path)
     parse_common_project_path = staticmethod(FirestoreClient.parse_common_project_path)
-
     common_location_path = staticmethod(FirestoreClient.common_location_path)
     parse_common_location_path = staticmethod(
         FirestoreClient.parse_common_location_path
@@ -93,7 +86,8 @@ class FirestoreAsyncClient:
 
     @classmethod
     def from_service_account_info(cls, info: dict, *args, **kwargs):
-        """Creates an instance of this client using the provided credentials info.
+        """Creates an instance of this client using the provided credentials
+            info.
 
         Args:
             info (dict): The service account private key info.
@@ -108,7 +102,7 @@ class FirestoreAsyncClient:
     @classmethod
     def from_service_account_file(cls, filename: str, *args, **kwargs):
         """Creates an instance of this client using the provided credentials
-        file.
+            file.
 
         Args:
             filename (str): The path to the service account private key json
@@ -125,7 +119,7 @@ class FirestoreAsyncClient:
 
     @property
     def transport(self) -> FirestoreTransport:
-        """Return the transport used by the client instance.
+        """Returns the transport used by the client instance.
 
         Returns:
             FirestoreTransport: The transport used by the client instance.
@@ -139,12 +133,12 @@ class FirestoreAsyncClient:
     def __init__(
         self,
         *,
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         transport: Union[str, FirestoreTransport] = "grpc_asyncio",
         client_options: ClientOptions = None,
         client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
     ) -> None:
-        """Instantiate the firestore client.
+        """Instantiates the firestore client.
 
         Args:
             credentials (Optional[google.auth.credentials.Credentials]): The
@@ -176,7 +170,6 @@ class FirestoreAsyncClient:
             google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                 creation failed for any reason.
         """
-
         self._client = FirestoreClient(
             credentials=credentials,
             transport=transport,
@@ -198,7 +191,6 @@ class FirestoreAsyncClient:
             request (:class:`google.cloud.firestore_v1.types.GetDocumentRequest`):
                 The request object. The request for
                 [Firestore.GetDocument][google.firestore.v1.Firestore.GetDocument].
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -212,7 +204,6 @@ class FirestoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-
         request = firestore.GetDocumentRequest(request)
 
         # Wrap the RPC method; this adds retry and timeout information,
@@ -224,9 +215,10 @@ class FirestoreAsyncClient:
                 maximum=60.0,
                 multiplier=1.3,
                 predicate=retries.if_exception_type(
-                    exceptions.DeadlineExceeded,
-                    exceptions.InternalServerError,
-                    exceptions.ServiceUnavailable,
+                    core_exceptions.DeadlineExceeded,
+                    core_exceptions.InternalServerError,
+                    core_exceptions.ResourceExhausted,
+                    core_exceptions.ServiceUnavailable,
                 ),
                 deadline=60.0,
             ),
@@ -260,7 +252,6 @@ class FirestoreAsyncClient:
             request (:class:`google.cloud.firestore_v1.types.ListDocumentsRequest`):
                 The request object. The request for
                 [Firestore.ListDocuments][google.firestore.v1.Firestore.ListDocuments].
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -277,7 +268,6 @@ class FirestoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-
         request = firestore.ListDocumentsRequest(request)
 
         # Wrap the RPC method; this adds retry and timeout information,
@@ -289,9 +279,10 @@ class FirestoreAsyncClient:
                 maximum=60.0,
                 multiplier=1.3,
                 predicate=retries.if_exception_type(
-                    exceptions.DeadlineExceeded,
-                    exceptions.InternalServerError,
-                    exceptions.ServiceUnavailable,
+                    core_exceptions.DeadlineExceeded,
+                    core_exceptions.InternalServerError,
+                    core_exceptions.ResourceExhausted,
+                    core_exceptions.ServiceUnavailable,
                 ),
                 deadline=60.0,
             ),
@@ -355,7 +346,6 @@ class FirestoreAsyncClient:
                 This corresponds to the ``update_mask`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -382,7 +372,6 @@ class FirestoreAsyncClient:
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
-
         if document is not None:
             request.document = document
         if update_mask is not None:
@@ -396,7 +385,10 @@ class FirestoreAsyncClient:
                 initial=0.1,
                 maximum=60.0,
                 multiplier=1.3,
-                predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                predicate=retries.if_exception_type(
+                    core_exceptions.ResourceExhausted,
+                    core_exceptions.ServiceUnavailable,
+                ),
                 deadline=60.0,
             ),
             default_timeout=60.0,
@@ -440,7 +432,6 @@ class FirestoreAsyncClient:
                 This corresponds to the ``name`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -461,7 +452,6 @@ class FirestoreAsyncClient:
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
-
         if name is not None:
             request.name = name
 
@@ -474,9 +464,10 @@ class FirestoreAsyncClient:
                 maximum=60.0,
                 multiplier=1.3,
                 predicate=retries.if_exception_type(
-                    exceptions.DeadlineExceeded,
-                    exceptions.InternalServerError,
-                    exceptions.ServiceUnavailable,
+                    core_exceptions.DeadlineExceeded,
+                    core_exceptions.InternalServerError,
+                    core_exceptions.ResourceExhausted,
+                    core_exceptions.ServiceUnavailable,
                 ),
                 deadline=60.0,
             ),
@@ -511,7 +502,6 @@ class FirestoreAsyncClient:
             request (:class:`google.cloud.firestore_v1.types.BatchGetDocumentsRequest`):
                 The request object. The request for
                 [Firestore.BatchGetDocuments][google.firestore.v1.Firestore.BatchGetDocuments].
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -525,7 +515,6 @@ class FirestoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-
         request = firestore.BatchGetDocumentsRequest(request)
 
         # Wrap the RPC method; this adds retry and timeout information,
@@ -537,9 +526,10 @@ class FirestoreAsyncClient:
                 maximum=60.0,
                 multiplier=1.3,
                 predicate=retries.if_exception_type(
-                    exceptions.DeadlineExceeded,
-                    exceptions.InternalServerError,
-                    exceptions.ServiceUnavailable,
+                    core_exceptions.DeadlineExceeded,
+                    core_exceptions.InternalServerError,
+                    core_exceptions.ResourceExhausted,
+                    core_exceptions.ServiceUnavailable,
                 ),
                 deadline=300.0,
             ),
@@ -581,7 +571,6 @@ class FirestoreAsyncClient:
                 This corresponds to the ``database`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -608,7 +597,6 @@ class FirestoreAsyncClient:
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
-
         if database is not None:
             request.database = database
 
@@ -621,9 +609,10 @@ class FirestoreAsyncClient:
                 maximum=60.0,
                 multiplier=1.3,
                 predicate=retries.if_exception_type(
-                    exceptions.DeadlineExceeded,
-                    exceptions.InternalServerError,
-                    exceptions.ServiceUnavailable,
+                    core_exceptions.DeadlineExceeded,
+                    core_exceptions.InternalServerError,
+                    core_exceptions.ResourceExhausted,
+                    core_exceptions.ServiceUnavailable,
                 ),
                 deadline=60.0,
             ),
@@ -674,7 +663,6 @@ class FirestoreAsyncClient:
                 This corresponds to the ``writes`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -701,10 +689,8 @@ class FirestoreAsyncClient:
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
-
         if database is not None:
             request.database = database
-
         if writes:
             request.writes.extend(writes)
 
@@ -716,7 +702,10 @@ class FirestoreAsyncClient:
                 initial=0.1,
                 maximum=60.0,
                 multiplier=1.3,
-                predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                predicate=retries.if_exception_type(
+                    core_exceptions.ResourceExhausted,
+                    core_exceptions.ServiceUnavailable,
+                ),
                 deadline=60.0,
             ),
             default_timeout=60.0,
@@ -765,7 +754,6 @@ class FirestoreAsyncClient:
                 This corresponds to the ``transaction`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -786,7 +774,6 @@ class FirestoreAsyncClient:
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
-
         if database is not None:
             request.database = database
         if transaction is not None:
@@ -801,9 +788,10 @@ class FirestoreAsyncClient:
                 maximum=60.0,
                 multiplier=1.3,
                 predicate=retries.if_exception_type(
-                    exceptions.DeadlineExceeded,
-                    exceptions.InternalServerError,
-                    exceptions.ServiceUnavailable,
+                    core_exceptions.DeadlineExceeded,
+                    core_exceptions.InternalServerError,
+                    core_exceptions.ResourceExhausted,
+                    core_exceptions.ServiceUnavailable,
                 ),
                 deadline=60.0,
             ),
@@ -836,7 +824,6 @@ class FirestoreAsyncClient:
             request (:class:`google.cloud.firestore_v1.types.RunQueryRequest`):
                 The request object. The request for
                 [Firestore.RunQuery][google.firestore.v1.Firestore.RunQuery].
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -850,7 +837,6 @@ class FirestoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-
         request = firestore.RunQueryRequest(request)
 
         # Wrap the RPC method; this adds retry and timeout information,
@@ -862,9 +848,10 @@ class FirestoreAsyncClient:
                 maximum=60.0,
                 multiplier=1.3,
                 predicate=retries.if_exception_type(
-                    exceptions.DeadlineExceeded,
-                    exceptions.InternalServerError,
-                    exceptions.ServiceUnavailable,
+                    core_exceptions.DeadlineExceeded,
+                    core_exceptions.InternalServerError,
+                    core_exceptions.ResourceExhausted,
+                    core_exceptions.ServiceUnavailable,
                 ),
                 deadline=300.0,
             ),
@@ -902,7 +889,6 @@ class FirestoreAsyncClient:
             request (:class:`google.cloud.firestore_v1.types.PartitionQueryRequest`):
                 The request object. The request for
                 [Firestore.PartitionQuery][google.firestore.v1.Firestore.PartitionQuery].
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -919,7 +905,6 @@ class FirestoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-
         request = firestore.PartitionQueryRequest(request)
 
         # Wrap the RPC method; this adds retry and timeout information,
@@ -931,9 +916,10 @@ class FirestoreAsyncClient:
                 maximum=60.0,
                 multiplier=1.3,
                 predicate=retries.if_exception_type(
-                    exceptions.DeadlineExceeded,
-                    exceptions.InternalServerError,
-                    exceptions.ServiceUnavailable,
+                    core_exceptions.DeadlineExceeded,
+                    core_exceptions.InternalServerError,
+                    core_exceptions.ResourceExhausted,
+                    core_exceptions.ServiceUnavailable,
                 ),
                 deadline=300.0,
             ),
@@ -1051,9 +1037,10 @@ class FirestoreAsyncClient:
                 maximum=60.0,
                 multiplier=1.3,
                 predicate=retries.if_exception_type(
-                    exceptions.DeadlineExceeded,
-                    exceptions.InternalServerError,
-                    exceptions.ServiceUnavailable,
+                    core_exceptions.DeadlineExceeded,
+                    core_exceptions.InternalServerError,
+                    core_exceptions.ResourceExhausted,
+                    core_exceptions.ServiceUnavailable,
                 ),
                 deadline=86400.0,
             ),
@@ -1095,7 +1082,6 @@ class FirestoreAsyncClient:
                 This corresponds to the ``parent`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -1125,7 +1111,6 @@ class FirestoreAsyncClient:
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
-
         if parent is not None:
             request.parent = parent
 
@@ -1138,9 +1123,10 @@ class FirestoreAsyncClient:
                 maximum=60.0,
                 multiplier=1.3,
                 predicate=retries.if_exception_type(
-                    exceptions.DeadlineExceeded,
-                    exceptions.InternalServerError,
-                    exceptions.ServiceUnavailable,
+                    core_exceptions.DeadlineExceeded,
+                    core_exceptions.InternalServerError,
+                    core_exceptions.ResourceExhausted,
+                    core_exceptions.ServiceUnavailable,
                 ),
                 deadline=60.0,
             ),
@@ -1190,7 +1176,6 @@ class FirestoreAsyncClient:
             request (:class:`google.cloud.firestore_v1.types.BatchWriteRequest`):
                 The request object. The request for
                 [Firestore.BatchWrite][google.firestore.v1.Firestore.BatchWrite].
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -1204,7 +1189,6 @@ class FirestoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-
         request = firestore.BatchWriteRequest(request)
 
         # Wrap the RPC method; this adds retry and timeout information,
@@ -1216,7 +1200,9 @@ class FirestoreAsyncClient:
                 maximum=60.0,
                 multiplier=1.3,
                 predicate=retries.if_exception_type(
-                    exceptions.Aborted, exceptions.ServiceUnavailable,
+                    core_exceptions.Aborted,
+                    core_exceptions.ResourceExhausted,
+                    core_exceptions.ServiceUnavailable,
                 ),
                 deadline=60.0,
             ),
@@ -1250,7 +1236,6 @@ class FirestoreAsyncClient:
             request (:class:`google.cloud.firestore_v1.types.CreateDocumentRequest`):
                 The request object. The request for
                 [Firestore.CreateDocument][google.firestore.v1.Firestore.CreateDocument].
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -1264,7 +1249,6 @@ class FirestoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-
         request = firestore.CreateDocumentRequest(request)
 
         # Wrap the RPC method; this adds retry and timeout information,
@@ -1275,7 +1259,10 @@ class FirestoreAsyncClient:
                 initial=0.1,
                 maximum=60.0,
                 multiplier=1.3,
-                predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                predicate=retries.if_exception_type(
+                    core_exceptions.ResourceExhausted,
+                    core_exceptions.ServiceUnavailable,
+                ),
                 deadline=60.0,
             ),
             default_timeout=60.0,
