@@ -18,6 +18,8 @@ import types
 import aiounittest
 
 import mock
+from google.cloud.firestore_v1.types.document import Document
+from google.cloud.firestore_v1.types.firestore import RunQueryResponse
 from tests.unit.v1.test__helpers import AsyncIter, AsyncMock
 
 
@@ -373,6 +375,125 @@ class TestAsyncClient(aiounittest.AsyncTestCase):
             metadata=client._rpc_metadata,
         )
 
+    def test_bulk_writer(self):
+        """BulkWriter is opaquely async and thus does not have a dedicated
+        async variant."""
+        from google.cloud.firestore_v1.bulk_writer import BulkWriter
+
+        client = self._make_default_one()
+        bulk_writer = client.bulk_writer()
+        self.assertIsInstance(bulk_writer, BulkWriter)
+        self.assertIs(bulk_writer._client, client._sync_copy)
+
+    def test_sync_copy(self):
+        client = self._make_default_one()
+        # Multiple calls to this method should return the same cached instance.
+        self.assertIs(client._to_sync_copy(), client._to_sync_copy())
+
+    @pytest.mark.asyncio
+    async def test_recursive_delete(self):
+        client = self._make_default_one()
+        client._firestore_api_internal = AsyncMock(spec=["run_query"])
+        collection_ref = client.collection("my_collection")
+
+        results = []
+        for index in range(10):
+            results.append(
+                RunQueryResponse(document=Document(name=f"{collection_ref.id}/{index}"))
+            )
+
+        chunks = [
+            results[:3],
+            results[3:6],
+            results[6:9],
+            results[9:],
+        ]
+
+        def _get_chunk(*args, **kwargs):
+            return AsyncIter(items=chunks.pop(0))
+
+        client._firestore_api_internal.run_query.side_effect = _get_chunk
+
+        bulk_writer = mock.MagicMock()
+        bulk_writer.mock_add_spec(spec=["delete", "close"])
+
+        num_deleted = await client.recursive_delete(
+            collection_ref, bulk_writer=bulk_writer, chunk_size=3
+        )
+        self.assertEqual(num_deleted, len(results))
+
+    @pytest.mark.asyncio
+    async def test_recursive_delete_from_document(self):
+        client = self._make_default_one()
+        client._firestore_api_internal = mock.Mock(
+            spec=["run_query", "list_collection_ids"]
+        )
+        collection_ref = client.collection("my_collection")
+
+        collection_1_id: str = "collection_1_id"
+        collection_2_id: str = "collection_2_id"
+
+        parent_doc = collection_ref.document("parent")
+
+        collection_1_results = []
+        collection_2_results = []
+
+        for index in range(10):
+            collection_1_results.append(
+                RunQueryResponse(document=Document(name=f"{collection_1_id}/{index}"),),
+            )
+
+            collection_2_results.append(
+                RunQueryResponse(document=Document(name=f"{collection_2_id}/{index}"),),
+            )
+
+        col_1_chunks = [
+            collection_1_results[:3],
+            collection_1_results[3:6],
+            collection_1_results[6:9],
+            collection_1_results[9:],
+        ]
+
+        col_2_chunks = [
+            collection_2_results[:3],
+            collection_2_results[3:6],
+            collection_2_results[6:9],
+            collection_2_results[9:],
+        ]
+
+        async def _get_chunk(*args, **kwargs):
+            start_at = (
+                kwargs["request"]["structured_query"].start_at.values[0].reference_value
+            )
+
+            if collection_1_id in start_at:
+                return AsyncIter(col_1_chunks.pop(0))
+            return AsyncIter(col_2_chunks.pop(0))
+
+        async def _get_collections(*args, **kwargs):
+            return AsyncIter([collection_1_id, collection_2_id])
+
+        client._firestore_api_internal.run_query.side_effect = _get_chunk
+        client._firestore_api_internal.list_collection_ids.side_effect = (
+            _get_collections
+        )
+
+        bulk_writer = mock.MagicMock()
+        bulk_writer.mock_add_spec(spec=["delete", "close"])
+
+        num_deleted = await client.recursive_delete(
+            parent_doc, bulk_writer=bulk_writer, chunk_size=3
+        )
+
+        expected_len = len(collection_1_results) + len(collection_2_results) + 1
+        self.assertEqual(num_deleted, expected_len)
+
+    @pytest.mark.asyncio
+    async def test_recursive_delete_raises(self):
+        client = self._make_default_one()
+        with self.assertRaises(TypeError):
+            await client.recursive_delete(object())
+
     def test_batch(self):
         from google.cloud.firestore_v1.async_batch import AsyncWriteBatch
 
